#!/usr/bin/perl -W

# 
# 
# ________     ___ ___  _________  __________ .__
# \______ \   /   |   \ \_   ___ \ \______   \|__|  ____     ____
#  |    |  \ /    ~    \/    \  \/  |     ___/|  | /    \   / ___\
#  |    `   \\    Y    /\     \____ |    |    |  ||   |  \ / /_/  >
# /_______  / \___|_  /  \______  / |____|    |__||___|  / \___  /
#         \/        \/          \/                     \/ /_____/


# Copyright (c) 2004, Gregory Duchemin <c3rb3r@sympatico.ca>
# All rights reserved.


# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# Redistributions of source code must retain the above copyright notice, this list of conditions
# and the following disclaimer.

# Redistributions in binary form must reproduce the above copyright notice, this list of conditions
# and the following disclaimer in the documentation and/or other materials provided with the distribution.

# Neither the name of the copyright holders nor the names of contributors may be used to endorse
# or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.





package Net::DHCPing::DHCP_CONST;
use strict;
use Net::DHCPing::DHCP_TOOLS qw(&inet_ntoa &inet_aton &sec2day);

BEGIN {
  use Exporter();
  use vars qw(@ISA @EXPORT_OK);
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(%DHCP_HTYPE %DHCP_OP %DHCP_FLAGS %DHCP_OPTIONS %DHCP_MSGTYPE %OPTIONS_DEC %OPTIONS_ENC %OPTIONS_ID);
}




#BOOTP/BOOTP operation
our %DHCP_OP=();

$DHCP_OP{0x01} = "BOOT REQUEST";
$DHCP_OP{0x02} = "BOOT REPLY";

#DHCP/BOOTP flags
our %DHCP_FLAGS=();

$DHCP_FLAGS{0x00} = "UNICAST";
$DHCP_FLAGS{0x01} = "BROADCAST";


#DHCP/BOOTP interface types
our %DHCP_HTYPE=();

$DHCP_HTYPE{0x01}="Ethernet (10Mb)";
$DHCP_HTYPE{0x02}="Experimental Ethernet (3Mb)";
$DHCP_HTYPE{0x03}="Amateur Radio AX.25";
$DHCP_HTYPE{0x04}="Proteon ProNET Token Ring";
$DHCP_HTYPE{0x05}="Chaos";
$DHCP_HTYPE{0x06}="IEEE 802 Networks";
$DHCP_HTYPE{0x07}="ARCNET";
$DHCP_HTYPE{0x08}="Hyperchannel";
$DHCP_HTYPE{0x09}="Lanstar";
$DHCP_HTYPE{0x0a}="Autonet Short Address";
$DHCP_HTYPE{0x0b}="LocalTalk";
$DHCP_HTYPE{0x0c}="LocalNet (IBM PCNet or SYTEK LocalNET)";
$DHCP_HTYPE{0x0d}="Ultra link";
$DHCP_HTYPE{0x0e}="SMDS";
$DHCP_HTYPE{0x0f}="Frame Relay";
$DHCP_HTYPE{0x10}="Asynchronous Transmission Mode (ATM)";
$DHCP_HTYPE{0x11}="HDLC";
$DHCP_HTYPE{0x12}="Fibre Channel";
$DHCP_HTYPE{0x13}="Asynchronous Transmission Mode (ATM)";
$DHCP_HTYPE{0x14}="Serial Line";
$DHCP_HTYPE{0x15}="Asynchronous Transmission Mode (ATM)";
$DHCP_HTYPE{0x16}="MIL-STD-188-220";
$DHCP_HTYPE{0x17}="Metricom";
$DHCP_HTYPE{0x18}="IEEE 1394.1995";
$DHCP_HTYPE{0x19}="MAPOS";
$DHCP_HTYPE{0x1a}="Twinaxial";
$DHCP_HTYPE{0x1b}="EUI-64";
$DHCP_HTYPE{0x1c}="HIPARP";
$DHCP_HTYPE{0x1d}="IP and ARP over ISO 7816-3";
$DHCP_HTYPE{0x1e}="ARPSec";
$DHCP_HTYPE{0x1f}="IPsec tunnel";
$DHCP_HTYPE{0x20}="Infiniband";


#DHCP/BOOTP options identification
our %OPTIONS_ID=();

$OPTIONS_ID{LEASE_TIME}="DHCP ";
$OPTIONS_ID{TIME_OFFSET}="BOOTP";
$OPTIONS_ID{OVERLOAD}="DHCP ";
$OPTIONS_ID{MESSAGE_TYPE}="DHCP ";
$OPTIONS_ID{SERVER_ID}="DHCP ";
$OPTIONS_ID{PARAMETERS}="DHCP ";
$OPTIONS_ID{MESSAGE}="DHCP ";
$OPTIONS_ID{MSGSIZE_MAX}="DHCP ";
$OPTIONS_ID{BOOTFILE}="BOOTP";
$OPTIONS_ID{CLIENT_ID}="DHCP ";
$OPTIONS_ID{LOG_SERVER}="DHCP ";
$OPTIONS_ID{REQUEST_IP}="DHCP ";
$OPTIONS_ID{RENEW}="DHCP ";
$OPTIONS_ID{REBIND}="DHCP ";
$OPTIONS_ID{SUBNET_MASK}="BOOTP";
$OPTIONS_ID{ROUTER}="BOOTP";
$OPTIONS_ID{DOMAIN}="BOOTP";
$OPTIONS_ID{DNS_SERVER}="BOOTP";
$OPTIONS_ID{HOSTNAME}="BOOTP";
$OPTIONS_ID{RESERVED}="BOOTP";
$OPTIONS_ID{CLASS_ID}="DHCP ";
$OPTIONS_ID{USER_CLASS_ID}="DHCP ";
$OPTIONS_ID{CLASS_SPECIFIC}="BOOTP";
$OPTIONS_ID{INVALID}="BOOTP";
$OPTIONS_ID{FQDN}="DHCP ";
$OPTIONS_ID{NETBIOS_SCOPE_ID}="DHCP ";
$OPTIONS_ID{NETBIOS_TCPIP_NS}="DHCP ";
$OPTIONS_ID{AUTHENTICATION}="BOOTP";
$OPTIONS_ID{CLIENT_SYSTEM}="BOOTP";
$OPTIONS_ID{CLIENT_DEVICE}="BOOTP";
$OPTIONS_ID{AUTOCONFIG}="BOOTP";
$OPTIONS_ID{RELAY_AGENT_INFO}="DHCP ";
$OPTIONS_ID{BROADCAST_ADDRESS}="BOOTP";


# DHCP MESSAGE TYPE option
our %DHCP_MSGTYPE=();

$DHCP_MSGTYPE{DISCOVER} = 0x01;
$DHCP_MSGTYPE{OFFER} = 0x02;
$DHCP_MSGTYPE{REQUEST} = 0x03;
$DHCP_MSGTYPE{DECLINE} = 0x04;
$DHCP_MSGTYPE{ACK} = 0x05;
$DHCP_MSGTYPE{NAK} = 0x06;
$DHCP_MSGTYPE{RELEASE} = 0x07;
$DHCP_MSGTYPE{INFORM} = 0x08;
$DHCP_MSGTYPE{FORCERENEW} = 0x09;
$DHCP_MSGTYPE{ILLEGAL} = 0xfe;
$DHCP_MSGTYPE{NONE} = 0xff;



# DHCP/BOOTP options and associated encoding/decoding functions
our %DHCP_OPTIONS=();
our %OPTIONS_DEC=();
our %OPTIONS_ENC=();


$DHCP_OPTIONS{LEASE_TIME}=0x33;
$OPTIONS_DEC{0x33} = sub {
                        return sec2day(unpack("N", shift()));
};
$OPTIONS_ENC{0x33} = sub {
                        return pack("N", shift());
};


$DHCP_OPTIONS{TIME_OFFSET}=0x02;
$OPTIONS_DEC{0x02} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x02} = sub {
                    return pack("C*", shift());
};


$DHCP_OPTIONS{OVERLOAD}=0x34;
$OPTIONS_DEC{0x34} = sub {
                    my $ovload =  unpack("H*", shift());
                    my $status = "";
                    ($ovload == 1)?$status = "file":(($ovload == 2)?$status = "sname":$status = "file sname");
                    return sprintf("0x%s (%s)", $ovload, $status);
};
$OPTIONS_ENC{0x34} = sub {
                    return pack("C*", shift());
};


$DHCP_OPTIONS{MESSAGE_TYPE}=0x35;
$OPTIONS_DEC{0x35} = sub {
                        my %REV_TYPE = reverse %DHCP_MSGTYPE;
                        return $REV_TYPE{unpack("C", shift())};
};
$OPTIONS_ENC{0x35} = sub {
                    return pack('C', $DHCP_MSGTYPE{uc(shift())})
};


$DHCP_OPTIONS{SERVER_ID}=0x36;
$OPTIONS_DEC{0x36} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x36} = sub {
                    return inet_aton(shift());
};

$DHCP_OPTIONS{NETBIOS_TCPIP_NS}=0x2c;
$OPTIONS_DEC{0x2c} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x2c} = sub {
                    return inet_aton(shift());
};

$DHCP_OPTIONS{PARAMETERS}=0x37;
$OPTIONS_DEC{0x37} = sub {
                    my @opts = unpack('C*', shift());
                    my @ret = ();
                    my %REV_OPTIONS = reverse %DHCP_OPTIONS; 
                    foreach my $opt (@opts) {
                      push @ret, (exists $REV_OPTIONS{$opt}?$REV_OPTIONS{$opt}:"Unknown") . " (" . $opt . " [" . sprintf("0x%02x", $opt) . "])  ";  
                    }
                    return @ret;
};
$OPTIONS_ENC{0x37} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{MESSAGE}=0x38;
$OPTIONS_DEC{0x38} = sub {
                    return shift();
};
$OPTIONS_ENC{0x38} = sub {
                    return pack("A*", shift());
};

$DHCP_OPTIONS{NETBIOS_SCOPE_ID}=0x2f;
$OPTIONS_DEC{0x2f} = sub {
                    return shift();
};
$OPTIONS_ENC{0x2f} = sub {
                    return pack("A*", shift());
};

$DHCP_OPTIONS{MSGSIZE_MAX}=0x39;
$OPTIONS_DEC{0x39} = sub {
                    return sprintf("%d", unpack("N", shift()));
};
$OPTIONS_ENC{0x39} = sub {
                    return pack("N", shift());
};


$DHCP_OPTIONS{BOOTFILE}=0x43;
$OPTIONS_DEC{0x43} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x43} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{CLIENT_ID}=0x3d;
$OPTIONS_DEC{0x3d} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x3d} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{USER_CLASS_ID}=0x4d;
$OPTIONS_DEC{0x4d} = sub {
                    return sprintf("%s", shift());
};
$OPTIONS_ENC{0x4d} = sub {
                    return pack("A*", shift());
};

$DHCP_OPTIONS{REQUEST_IP}=0x32;
$OPTIONS_DEC{0x32} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x32} = sub {
                    return inet_aton(shift());
};

$DHCP_OPTIONS{LOG_SERVER}=0x07;
$OPTIONS_DEC{0x07} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x07} = sub {
                    return inet_aton(shift());
};


$DHCP_OPTIONS{RENEW}=0x3a;
$OPTIONS_DEC{0x3a} = sub {
                        return sec2day(unpack("N", shift()));
};
$OPTIONS_ENC{0x3a} = sub {
                    return pack("N", shift());
};


$DHCP_OPTIONS{REBIND}=0x3b;
$OPTIONS_DEC{0x3b} = sub {
                        return sec2day(unpack("N", shift()));
};
$OPTIONS_ENC{0x3b} = sub {
                    return pack("N", shift());
};


$DHCP_OPTIONS{SUBNET_MASK}=0x01;
$OPTIONS_DEC{0x01} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x01} = sub {
                    return inet_aton(shift());
};


$DHCP_OPTIONS{BROADCAST_ADDRESS}=0x1c;
$OPTIONS_DEC{0x1c} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x1c} = sub {
                    return inet_aton(shift());
};


$DHCP_OPTIONS{ROUTER}=0x03;
$OPTIONS_DEC{0x03} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data) / 4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x03} = sub {
                    return inet_aton(shift());
};


$DHCP_OPTIONS{DOMAIN}=0x0f;
$OPTIONS_DEC{0x0f} = sub {
                    return shift();
};
$OPTIONS_ENC{0x0f} = sub {
                    return shift();
};


$DHCP_OPTIONS{DNS_SERVER}=0x06;
$OPTIONS_DEC{0x06} = sub {
                    my $data = shift;
                    defined $data?1:return '';
                    my $len = length($data)/4;
                    my @bytes = split //, $data;
                    $#bytes = ($len * 4) - 1;
                    my @ret = ();
                    for (my $i = 0; $i <= $#bytes; $i += 4) {
                      if ($i+3 <= $#bytes) {
                        push @ret, 
                            inet_ntoa(pack('aaaa', $bytes[$i], $bytes[$i+1], $bytes[$i+2], $bytes[$i+3]));
                      }
                    }
                    return @ret;
};
$OPTIONS_ENC{0x06} = sub {
                    return inet_aton(shift());
};


$DHCP_OPTIONS{HOSTNAME}=0x0c;
$OPTIONS_DEC{0x0c} = sub {
                    return shift();
};
$OPTIONS_ENC{0x0c} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{RESERVED}=0xfb;
$OPTIONS_DEC{0xfb} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0xfb} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{CLASS_ID}=0x3c;
$OPTIONS_DEC{0x3c} = sub {
                    return sprintf("%s", shift());
};
$OPTIONS_ENC{0x3c} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{CLASS_SPECIFIC}=0x2b;
$OPTIONS_DEC{0x2b} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x2b} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{INVALID}=0xfe;
$OPTIONS_DEC{0xfe} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0xfe} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{FQDN}=0x51;               # http://archive.cert.uni-stuttgart.de/in-drafts/draft-ietf-dhc-fqdn-option-03.txt
$OPTIONS_DEC{0x51} = sub {              # RFC 3046
            my ($FLAGS, $RCODE1, $RCODE2, $data) = unpack('CCCA*', shift());
            return sprintf("%s  (rcode1: %d  rcode2: %d  flags: %02x)", $data, $RCODE1, $RCODE2, $FLAGS);
};

$OPTIONS_ENC{0x51} = sub {              
            return pack("CCC", 0x01, 0, 0).pack("A*", shift());
};


$DHCP_OPTIONS{RELAY_AGENT_INFO}=0x52;
$OPTIONS_DEC{0x52} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x52} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{AUTHENTICATION}=0x5a;
$OPTIONS_DEC{0x5a} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x5a} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{CLIENT_SYSTEM}=0x5d;
$OPTIONS_DEC{0x5d} = sub {
                    return shift();
};
$OPTIONS_ENC{0x5d} = sub {
                    return shift();
};


$DHCP_OPTIONS{CLIENT_DEVICE}=0x5e;
$OPTIONS_DEC{0x5e} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x5e} = sub {
                    return pack("A*", shift());
};


$DHCP_OPTIONS{AUTOCONFIG}=0x74;
$OPTIONS_DEC{0x74} = sub {
                    return sprintf("0x%s", unpack("H*", shift()));
};
$OPTIONS_ENC{0x74} = sub {
                    return pack("A*", shift());
};







1;