#!/usr/bin/perl -W

# 
# 
# ________     ___ ___  _________  __________ .__
# \______ \   /   |   \ \_   ___ \ \______   \|__|  ____     ____
#  |    |  \ /    ~    \/    \  \/  |     ___/|  | /    \   / ___\
#  |    `   \\    Y    /\     \____ |    |    |  ||   |  \ / /_/  >
# /_______  / \___|_  /  \______  / |____|    |__||___|  / \___  /
#         \/        \/          \/                     \/ /_____/


# Copyright (c) 2004, Gregory Duchemin <c3rb3r@sympatico.ca>
# All rights reserved.


# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# Redistributions of source code must retain the above copyright notice, this list of conditions
# and the following disclaimer.

# Redistributions in binary form must reproduce the above copyright notice, this list of conditions
# and the following disclaimer in the documentation and/or other materials provided with the distribution.

# Neither the name of the copyright holders nor the names of contributors may be used to endorse
# or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Package contains statistic functions

package Net::DHCPing::DHCP_STATS;
use strict;
use Time::HiRes qw(tv_interval);
use Net::DHCPing::DHCP_TOOLS qw(&sec2day);

BEGIN {
  use Exporter();
  use vars qw(@ISA @EXPORT_OK);
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(&show_stats &show_listen_stats);
}



sub show_stats {
    my $interface =    shift;
    my @start_time = @{shift()};
    my @end_time =   @{shift()};
    my $c_in =         shift;
    my $c_out =        shift;
    my $c_timeout =    shift;
    
    my ($sent_av, $rec_av, $to_av) = (0, 0, 0);
    my $total_time = tv_interval(\@start_time, \@end_time);
    # avoid divisions by zero
    if ($total_time) {
        $sent_av =    sprintf("%.02f", $c_out / $total_time);
        $rec_av =     sprintf("%.02f", $c_in / $total_time);
        $to_av =      sprintf("%.02f", $c_timeout / $total_time);
        $total_time = sprintf("%.03f", $total_time);
    }
    
    print
        "\n",
        "\n", "Session started on ", &getdate(@start_time),
        "\n", "Session ended on ", &getdate(@end_time),
        "\n",
        "\n", "interface:             ", $interface,
        "\n", "request(s) sent:       ", $c_out,
        "\n", "response(s) received:  ", $c_in,
        "\n", "response(s) timeout:   ", $c_timeout,
        "\n", "total time:            ", &sec2day($total_time), " (", $total_time, " second(s))",
        "\n", "sent average:          ", $sent_av, " packet(s)/s",
        "\n", "received average:      ", $rec_av, " packet(s)/s",
        "\n", "timeout average:       ", $to_av, " packet(s)/s\n",
        "\n\n";
    return;
}


sub show_listen_stats {
    my $interface =    shift;
    my @start_time = @{shift()};
    my @end_time =   @{shift()};
    my $c_in =         shift;
    
    my $total_time = tv_interval(\@start_time, \@end_time);
    my $rec_av = 0;
    
    # avoid divisions by zero
    if ($total_time) {
        $rec_av =     sprintf("%.02f", $c_in / $total_time);
        $total_time = sprintf("%.03f", $total_time);
    }
    print
        "\n",
        "\n", "Session started on ", &getdate(@start_time),
        "\n", "Session ended on ", &getdate(@end_time),
        "\n",
        "\n", "interface:         ", $interface,
        "\n", "packet(s) caught:  ", $c_in,
        "\n", "total time:        ", &sec2day($total_time), " (", $total_time, " second(s))",
        "\n", "packet(s) average: ", $rec_av, " packet(s)/s\n",
        "\n\n";
}


sub getdate {
    my $time = shift;
    my @DayName = qw/Monday Thuesday Wednesday Thursday Friday Saturday Sunday/; 
    my ($Second, $Minute, $Hour, $Day, $Month, $Year, $WeekDay) = localtime($time);
    return  sprintf("%04d/%02d/%02d %s, %02d:%02d:%02d",
                    $Year + 1900,
                    ++$Month,
                    $Day,
                    $DayName[--$WeekDay],
                    $Hour,
                    $Minute,
                    $Second);
}


1;