#!/usr/bin/perl -W

# 
# 
# ________     ___ ___  _________  __________ .__
# \______ \   /   |   \ \_   ___ \ \______   \|__|  ____     ____
#  |    |  \ /    ~    \/    \  \/  |     ___/|  | /    \   / ___\
#  |    `   \\    Y    /\     \____ |    |    |  ||   |  \ / /_/  >
# /_______  / \___|_  /  \______  / |____|    |__||___|  / \___  /
#         \/        \/          \/                     \/ /_____/



# Copyright (c) 2004, Gregory Duchemin <c3rb3r@sympatico.ca>
# All rights reserved.


# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# Redistributions of source code must retain the above copyright notice, this list of conditions
# and the following disclaimer.

# Redistributions in binary form must reproduce the above copyright notice, this list of conditions
# and the following disclaimer in the documentation and/or other materials provided with the distribution.

# Neither the name of the copyright holders nor the names of contributors may be used to endorse
# or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


package Net::DHCPing::DHCP_TOOLS;
use strict;

BEGIN {
  use Exporter();
  use vars qw(@ISA @EXPORT_OK);
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(&mac2str &str2mac &pcap_mac2str &gen_randmac &sec2day &gen_string &inet_ntoa &inet_aton);
}



sub sec2day {
use integer;
my $time = shift();

my $day  = $time   / 86400;
my $hour = ($time  % 86400) / 3600;
my $min  = (($time % 86400) % 3600) / 60;
my $sec  = (($time % 86400) % 3600) % 60;

$time  = '';
$time .= $day." day(s) "   unless (!$day);
$time .= $hour." hour(s) " unless (!$hour);
$time .= $min." min(s) "   unless (!$min);
$time .= $sec." sec(s)"    unless (!$sec);
($time eq "")?$time = $sec . " sec":1;
return ($time);
}



## MAC addresses stuff

sub mac2str {
        my $len =  shift;
        my $addr = shift;
        if ((!defined $addr) || (!defined $len)) {return undef;}
        if ($len > 16) { return undef; }
	return join ':', unpack("H2" x $len, $addr);
}

sub pcap_mac2str {
        my $addr = shift;
        if (!defined $addr) {return undef;}
        my @ret = split //, $addr;
          
	return $ret[0].$ret[1].':'
              .$ret[2].$ret[3].':'
              .$ret[4].$ret[5].':'
              .$ret[6].$ret[7].':'
              .$ret[8].$ret[9].':'
              .$ret[10].$ret[11];
}

sub str2mac {
        my $len = shift;
        my $addr = shift;
        if ((!defined $addr) || (!defined $len)) {return undef;}
	return pack("H2" x $len, split(/:/, $addr));
}



# various random generators


sub gen_randmac {
  my $len = shift;
  my $addr = '';
  
   for (1 .. $len) {
        my $num = rand 256;
        $num /=         1;
        $addr .= sprintf("%02x:", $num);
    }
    chop($addr);
    return $addr;
}

sub gen_string {
    my $len = shift;
    my $string = '';
    
    for (1 .. $len) {
        my $num = rand 77;
        $num /= 1;
        $num += 48;
        $string .= chr($num);
    }
    return $string;
}



# private INET functions
# avoid the use of IO::Socket::INET and save an extra memory

sub inet_ntoa {
    my $addr = shift;
    my @addr = unpack("C4", $addr);
    
    return   $addr[0].'.'
            .$addr[1].'.'
            .$addr[2].'.'
            .$addr[3];
}

sub inet_aton {
    my $addr = shift;
    my @addr = split /\./, $addr;
    if ($#addr != 3) { return undef; }
    foreach (@addr) {
      if ($_ > 255) { return undef; }
    }
    return pack("C4", $addr[0], $addr[1], $addr[2], $addr[3]);
}

1;