#!/usr/bin/perl -W

# 
# 
# ________     ___ ___  _________  __________ .__
# \______ \   /   |   \ \_   ___ \ \______   \|__|  ____     ____
#  |    |  \ /    ~    \/    \  \/  |     ___/|  | /    \   / ___\
#  |    `   \\    Y    /\     \____ |    |    |  ||   |  \ / /_/  >
# /_______  / \___|_  /  \______  / |____|    |__||___|  / \___  /
#         \/        \/          \/                     \/ /_____/



# Copyright (c) 2004, Gregory Duchemin <c3rb3r@sympatico.ca>
# All rights reserved.


# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# Redistributions of source code must retain the above copyright notice, this list of conditions
# and the following disclaimer.

# Redistributions in binary form must reproduce the above copyright notice, this list of conditions
# and the following disclaimer in the documentation and/or other materials provided with the distribution.

# Neither the name of the copyright holders nor the names of contributors may be used to endorse
# or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Package displays usage


package Net::DHCPing::DHCP_USAGE;
use strict;

our $VERSION;
my $RELEASE_DATE;
our %COLOR;

BEGIN {
  use Exporter();
  use vars qw(@ISA @EXPORT_OK);
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(&usage &macros_usage %COLOR $VERSION);

  $COLOR{C_RED}     = "\033[0;31m";
  $COLOR{C_BRED}    = "\033[1;31m";
  $COLOR{C_YELLOW}  = "\033[0;33m";
  $COLOR{C_BYELLOW} = "\033[1;33m";
  $COLOR{C_GREEN}   = "\033[0;32m";
  $COLOR{C_BGREEN}  = "\033[1;32m";
  $COLOR{C_BLUE}    = "\033[0;34m";
  $COLOR{C_BBLUE}   = "\033[1;34m";
  $COLOR{C_WHITE}   = "\033[0;37m";
  $COLOR{C_BWHITE}  = "\033[1;37m";

  $VERSION = "0.90";
  $RELEASE_DATE =   "May 2004";
}


sub usage { 
    print
        "\n", &logo,
        "\nDHCPing, ".$VERSION." / ".$RELEASE_DATE."\n",
        "http://dhcping.openwall.net\n",
        "Copyrights (c) 2004, Gregory Duchemin <c3rb3r\@sympatico.ca>\n",
        $COLOR{C_BWHITE},
        "\nUSAGE:",
        $COLOR{C_WHITE},
        " dhcping [-h|-help]|[opts]\n\n",
        $COLOR{C_BWHITE},
        "\n  ETHERNET/IP\n",
        "---------------\n",
        $COLOR{C_WHITE},
        "-m|shaddr {addr}: source MAC address, (default : ff:ff:ff:ff:ff:ff)\n",
        "-M|dhaddr {addr}: destination MAC address, (default : ff:ff:ff:ff:ff:ff)\n",
        "-z|randsmac: randomize source MAC address\n",
        "-i|siaddr {addr}: source ip address, (default : 0.0.0.0)\n",
        "-I|diaddr {addr}: destination ip address, (default : 255.255.255.255)\n",
        "-p|sport {num}: source port, (default : 68/dhcp client)\n",
        "-P|dport {num}: destination port, (default : 67/dhcp server)\n",
        $COLOR{C_BWHITE},
        "\n  BOOTP header\n",
        "----------------\n",
        $COLOR{C_WHITE},
        "-o|bootop {num}: force bootp operator (default 1 (request))\n",
        "-showbootop: show valid bootp operators\n",
        "-e|htype {num}: force hardware type, (default: Ethernet 10 Mb/s)\n",
        "-s|secs {num}: force seconds field, (default: 1)\n",
        "-showhtypes: list available hardware types\n",
        "-l|hlen {num}: force hardware address length (default 6)\n",
        "-r|randxid: randomize xid, (default: incremental)\n",
        "-x|xid {num}: force xid, (default: 0)\n",
        "-H|hops {num}: set hops (default 0)\n",
        "-f|flags {num}: force flags field, (default 0x0000)\n",
        "-u|unicast: set unicast flag, (default)\n",
        "-b|broadcast: set broadcast flag\n",
        "-S|sname {string}: server name\n",
        "-F|file {string}: boot filename\n",
        "-g|giaddr {addr}: set 'gateway address' field\n",
        "-y|yiaddr {addr}: set 'your ip address' field\n",
        "-C|cookie {num}: force magic cookie, (default x63x82x53x63)\n",
        "-X|randcookie: randomize magic cookie\n",
        $COLOR{C_BWHITE},
        "\n  DHCP/BOOTP options\n",
        "----------------------\n",
        $COLOR{C_WHITE},
        "-opttype {string}: DHCP packet type, (defaut: none)\n",
        "-showopttypes: list available DHCP packet types\n",
        "-optreqip {addr}: use 'requested ip address' option, (default : none)\n",
        "-optreqallparams: use 'requested parameters' option and ask server to return all DHCP options (from 0x01 to 0xfe)\n",
        "-optleasetime {num}: use 'lease time' option in hours, (default: none)\n",
        "-optrouter {addr}: use 'router' option, (default: none)\n",
        "-optuserid {string}: use 'user class id' option, (default: none)\n",
        "-optnetmask {addr}: use 'subnet mask' option, (default: none)\n",
        "-optnbscopeid {string}: use 'netbios scope id' option, (default: none)\n",
        "-opthostname {string}: force 'hostname' option, (default: none)\n",
        "-randhostname {num}: randomize 'hostname' option with length {num}, (default: none)\n",
        "-optfqdn {string}: force 'fqdn' option, (default: none)\n",
        "-randfqdn {num}: randomize 'fqdn' option with length {num}, (default: none)\n",
        "-optdomain {string}: force 'domain' option, (default: none)\n",
        "-opttimeoffset {num}: force 'time offset' option, (default: none)\n",
        "-optclientsys {string}: force 'client system' option, (default: none)\n",
        "-optclassid {string}: force 'class id' option, (default: none)\n",
        "-optmessage {string}: force 'message' option, (default: none)\n",
        "-optmaxmsgsize {num}: max size requested for message option, (default: none)\n",
        "-optserverid: force 'server id' option, (default: destination ip address)\n",
        "-optclientid: include MAC based 'client id' option, (default: none)\n",
        "-optcustomclientid {string}: include forged 'client id' option, (default: none)\n",
        "-randclientid: randomize MAC based 'client id' option, (default: none)\n",
        "-optoverload {1/2/3}: 1/file 2/sname 3/both, (default: none)\n",
        "-optbootfile {string}: use 'boot file' option, (default: none)\n",
        "-showopts: list predefined DHCP/BOOTP options names/numbers\n",
        $COLOR{C_BWHITE},
        "\n  Misc\n",
        "--------\n",
        $COLOR{C_WHITE},
        "-n|nbr {num}: send {num} packets, -1 means loop forever, (default: 1)\n",
        "-interface {string}: specify a network interface, (default: 'eth0')\n",
        "-T|timeout {num}: response timeout in seconds, (default: 5 seconds)\n",
        "-listen: listen mode, no packet building, (default: none)\n",
        "-server: pseudo server mode, blindly reply with forged packets to clients, (default: none)\n",        
        "-W|wait {num}: wait for {num} response(s), 0 = don't wait, (default: 1)\n",
        "-pause {num}: pause {num} microseconds between each transmission, (default: 100000 (1/10s))\n",
        "-timing {polite|normal|aggressive|insane}: predefined pauses (see '-pause')\n",
        "-[no]v|verbose: [don't]display packets details(default: verbose)\n",
        "-[no]color: [don't]colorize output (default: color)\n",
        "-h|help: this text\n\n",
        $COLOR{C_BWHITE},
        "\n  Macro attacks\n",
        "-----------------\n",
        $COLOR{C_WHITE},
        "-showmacros: macros usage",
        "\n\n";

return;
}

sub macros_usage { 
    print
        "\n", &logo,
        "\nDHCPing, ".$VERSION ." / ".$RELEASE_DATE."\n",
        "http://dhcping.openwall.net\n",
        "Copyrights (c) 2004, Gregory Duchemin <c3rb3r\@sympatico.ca>\n",
        $COLOR{C_BWHITE},
        "\nUSAGE:",
        $COLOR{C_WHITE},
        " dhcping [-h|-help]|[opts]\n\n",
        $COLOR{C_BWHITE},
        "\n  Generic\n",
        "-----------\n",
        $COLOR{C_WHITE},
        "-halfexhaust {num}:    Storm of {num} DISCOVER requests (half negociation) from random MAC sources, exhausts DHCP address space\n",
        "                       choose {num} accordingly to the size of the address range you wish to exhaust, -1 loop forever\n",
        "                       tested against DLINK 704, DLINK 614+, DLINK 624\n",
        "-halfexhaust2 {num}:   Storm of {num} DISCOVER requests (half negociation), static MAC but random CLIENTID option\n",
        "                       choose {num} accordingly to the size of the address range you wish to exhaust, -1 loop forever\n",
        "                       tested against DLINK 614+, DLINK 624\n",
        "-fastexhaust {range}:  Storm of REQUEST requests (fast negociation, eg: ask for an ip wether it is free or not), \n",
        "                       {range} must be of the form 10.0.0 meaning a range from 10.0.0.1 to 10.0.0.254\n",
        "                       use -nbr to loop n time through the range, -nbr -1 loop forever\n",
        "                       tested against ISC DHCPD v3.0.1rc13, should work against most implementations around\n",
        "-declinestorm {range}: Storm of DECLINE requests (no negociation, eg: alert that ip's in {range} must be abandoned), \n",
        "                       {range} must be of the form 10.0.0 meaning a range from 10.0.0.1 to 10.0.0.254\n",
        "                       use -nbr to loop n time through the range, -nbr -1 loop forever\n",
        "                       tested against ISC DHCPD v3.0.1rc13\n",
        "-politexhaust {num}:   Flood of {num} DHCP full negociations (eg: ask for a free address (DISCOVER) and take it (REQUEST))\n",
        "                       choose {num} accordingly to the size of the address range you wish to exhaust, -1 loop forever\n",
        "                       tested against ISC DHCPD v3.0.1rc13, should work against most implementations around\n",
        "-corruptopt {num}:     Insert a corrupted (eg malformed/unfinished) option number {num}, (default: none)\n",
        "                       note that only one corrupted option can be sent per packet (-showopts for option numbers)\n",

        $COLOR{C_BWHITE},
        "\n  DLINK 6x4 serie tested against 614+ Rev A. firmware 2.30\n",
        "                                 624  Rev B. firmware 1.28\n",
        "------------------------------------------------------------\n",
        $COLOR{C_WHITE},
        "-6x4unlimited:       Lease time signedness bug, gives unlimited lease time (13.5 years, almost :)\n",
        "                     may become a permanent dos in an exhaustion attack, ie: dhcpstorm -6x4unlimited -halfexhaust 200\n",
        "-6x4expel1 {mac}:    Another lease time signedness bug, expels {mac} current lease\n",
        "                     note that the server will actually check {mac} address in the dhcp 'clientid' option\n",
        "                     also note that killing leases this way may initiate a subsequent dhcp chaos\n",
        "-6x4expel2 {mac}:    Erroneous REQUEST, not including a REQUEST IP option, expels {mac} current lease \n",
        "                     note that the server will actually check {mac} address in the dhcp 'clientid' option\n",
        "                     also note that killing leases this way may initiate a subsequent dhcp chaos\n",
        "-6x4xss {addr}:      Trigger 'XSS over DHCP Hostname' hole in {addr} server, then look at the dhcp and log pages\n",
        "-6x4xss2 {addr}:     Exploit previous hole, force {addr} to reboot as soon as dhcp or log pages are open\n",
        "                     with either msie or netscape\n",
        "-6x4xss3 {addr}:     Yet another exploit for {addr} previous hole, this time restores DLINK default settings,\n",
        "                     ip addr 192.168.0.1, blank password, no WEP etc... :)\n",
        "-6x4memleak1 {addr}: Memory leak in {addr} server while handling a corrupted HOSTNAME option in a DHCP REQUEST\n",
        "                     raw memory chunks are visible in the DHCP page\n",
        "-6x4memleak2:        Memory leak while handling a corrupted CLIENT_ID option in DHCP DISCOVER/REQUEST\n",
        "                     raw memory chunks (asciihex encoded) are visible in the DHCP page\n",
        "-6x4reboot {addr}:   Flood {addr} server with forged MAC addresses, finally deplete memory and trigger a reboot\n",
        "                     note that a reboot for a DI6x4 also means the loss of leases and logs, so a subsequent dhcp chaos may appear\n",
        "                     this bug seems to be related to memory exhaustion while processing a high number of simultaneous requests afterwhat...\n",
        $COLOR{C_BWHITE},
        "\n  DLINK 704 tested against firmware 2.60 build 2\n",
        "--------------------------------------------------\n",
        $COLOR{C_WHITE},
        "-704xss {addr}:  'XSS over DHCP Hostname' in DLINK 'DHCP/fixed mapping' page, request must include -I server_addr\n",
        "-704xss2 {addr}: Exploit previous hole with a fake 'trojan' relogin page, bad bad bad ;), request must include -I server_addr\n",
        "                 the fake page is actually hosted in mother Russia, you need Internet for this one.\n",
        "-704xss3 {addr}: Enable SNMP admin on both external and internal sides.\n",
        "                 Also set 'public'/'private' as read and write communities, request must include -I server_addr\n",
        $COLOR{C_BWHITE},
        "\n  INFOBLOX DNSONE tested against feature set <= 2.4.0-8\n",
        "---------------------------------------------------------\n",
        $COLOR{C_WHITE},
        "-dns1xss {addr}:  'XSS over DHCP Hostname' in DNSONE 'DHCP report' page\n",
        "                  display a nice smiley and a short message over the DHCP report.\n",
        "                  note that it will be displayed twice if dynamic update is on, because of the same hole\n",
        "                  in the unfiltered 'ddns-fwd-name' field.\n",
        "-dns1xss2 {addr}: 'XSS over DHCP Clientid' in DNSONE 'DHCP report' page\n",
        "                  open a javascript popup when opening DHCP report.\n",
        $COLOR{C_BWHITE},
        "\n  ISC DHCPD 3.0.1 rc12/13\n",
        "---------------------------\n",
        $COLOR{C_WHITE},
	"-iscexpel {addr}: Send a decline request and force ISC server to abandon {addr}\n",
        "                  free or already leased {addr} will be droped from the active pool\n",
        "                  note this doesn't require any form of identification, (client id, mac address ...)\n",
        "                  for address already used. Therefore a subsequent dhcp chaos may occur\n",
        "-iscdos {addr}:   Denial of service against ISC DHCPD, sending an overlong hostname\n",
        "                  splitted in multiple chunk as allowed by RFC 3396\n",
        "\n\n";
return;
}


# Figlet is cool, but causes a lot of warnings in -w mode
# plus by removing it people won t have to get it installed
# also that saves an extra and very modest chunk of memory
# this logo was however produced with it and copied after that

sub logo {

return "\n", $COLOR{C_GREEN},
"                                                                   \n", $COLOR{C_BGREEN},
" ________     ___ ___  _________  __________ .__\n",
" \\______ \\   /   |   \\ \\_   ___ \\ \\______   \\|__|  ____     ____\n", $COLOR{C_GREEN},
"  |    |  \\ /    ~    \\/    \\  \\/  |     ___/|  | /    \\   / ___\\\n",
"  |    `   \\\\    Y    /\\     \\____ |    |    |  ||   |  \\ / /_/  >\n", $COLOR{C_BLUE},
" /_______  / \\___|_  /  \\______  / |____|    |__||___|  / \\___  /\n", 
"         \\/        \\/          \\/                     \\/ /_____/\n", $COLOR{C_GREEN};
}


1;